@@ -1,4 +1,4 @@
-// $Id: QUIC.C,v 1.53 2012-05-17 02:46:53 sustik Exp $
+// $Id: QUIC.C,v 1.41 2012-02-28 22:19:54 sustik Exp $
 
 // Solves the regularized inverse covariance matrix selection using a
 // combination of Newton's method, quadratic approximation and
@@ -9,7 +9,7 @@
 // See the README file and QUIC.m for more information.
 // Send questions, comments and license inquiries to: sustik@cs.utexas.edu
 
-#define VERSION "1.1"
+#define VERSION "2012.02"
 
 #include <math.h>
 #include <stddef.h>
@@ -26,7 +26,7 @@
   #include <R_ext/Print.h>
   #define MSG Rprintf
 #endif
-#if defined(LANG_M) || defined(MATLAB_MEX_FILE)
+#ifdef LANG_M
   #include <mex.h>
   #define MSG mexPrintf
 #endif
@@ -38,10 +38,10 @@
 #define EPS (double(2.22E-16))
 //#define EPS ((double)0x3cb0000000000000)
 
-#if !defined(LANG_R) && defined(_WIN32)
-  #define dpotrf_ dpotrf
-  #define dpotri_ dpotri
-#endif
+typedef struct {
+    unsigned short i;
+    unsigned short j;
+} ushort_pair_t;
 
 // It would be preferable to use an include such as lapack.h.  Except
 // lapack.h is not available from the octave or liblapack-dev packages...
@@ -52,11 +52,6 @@ extern "C" {
 		 ptrdiff_t* info);
 }
 
-typedef struct {
-    unsigned short i;
-    unsigned short j;
-} ushort_pair_t;
-
 static inline unsigned long IsDiag(unsigned long p, const double* A)
 {
     for (unsigned long k = 0, i = 0; i < p; i++, k += p)
@@ -186,35 +181,8 @@ static inline double DiagNewton(unsigned long p, const double* S,
     return fX;
 }
 
-static double projLogDet(unsigned long p, const double* S,
-			 double* W, double* prW, double* Lambda)
-{
-    // The computed W does not satisfy |W - S| .< Lambda.  Project it.
-    for (unsigned long i = 0; i < p; i++) {
-	for (unsigned long j = 0; j <= i; j++) {
-	    double tmp = W[i*p+j];
-	    if (S[i*p+j] - Lambda[i*p+j] > tmp)
-		tmp = S[i*p+j] - Lambda[i*p+j];
-	    if (S[i*p+j] + Lambda[i*p+j] < tmp)
-		tmp = S[i*p+j] + Lambda[i*p+j];
-	    prW[i*p+j] = tmp;
-	    prW[j*p+i] = tmp;
-	}
-    }
-    ptrdiff_t info = 0;
-    ptrdiff_t p0 = p;
-    dpotrf_((char*) "U", &p0, prW, &p0, &info);
-    if (info != 0)
-	return 1e+15;
-    double logdet = 0.0;
-    for (unsigned long i = 0, k = 0; i < p; i++, k += (p+1))
-	logdet += log(prW[k]);
-    logdet += logdet;
-    return logdet;
-}
-
 #define QUIC_MSG_NO      0
-#define QUIC_MSG_MIN     1
+#define QUIC_MSG_TIME    1
 #define QUIC_MSG_NEWTON  2
 #define QUIC_MSG_CD      3
 #define QUIC_MSG_LINE    4
@@ -223,9 +191,8 @@ static double projLogDet(unsigned long p, const double* S,
 extern "C"
 void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	  uint32_t& pathLen, const double* path, double& tol,
-	  int32_t& msg, uint32_t& maxIter,
-	  double* X, double* W, double* opt, double* cputime,
-	  uint32_t* iter, double* dGap)
+	  int32_t& msg, uint32_t* iter, double* X,
+	  double* W, double* opt, double* cputime)
 {
 #ifdef GDEBUG
     startgdb();
@@ -233,7 +200,7 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
     if (mode >= 'a')
 	mode -= ('a' - 'A');
     char m[] = "Running QUIC version";
-    if (msg >= QUIC_MSG_MIN) {
+    if (msg >= QUIC_MSG_TIME) {
 	if (mode == 'P')
 	    MSG("%s %s in 'path' mode.\n", m, VERSION);
 	else if (mode == 'T')
@@ -243,7 +210,7 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
     }
     double timeBegin = clock();
     srand(1);
-    unsigned long maxNewtonIter = maxIter;
+    unsigned long maxNewtonIter = iter[0];
     double cdSweepTol = 0.05;
     unsigned long max_lineiter = 20;
     double fX = 1e+15;
@@ -251,6 +218,7 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
     double fXprev = 1e+15;
     double sigma = 0.001;
     double* D = (double*) malloc(p*p*sizeof(double));
+    double* X1 = (double*) malloc(p*p*sizeof(double));
     double* U = (double*) malloc(p*p*sizeof(double));
     double* Lambda;
     if (pathLen > 1) {
@@ -264,7 +232,6 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	malloc(p*(p+1)/2*sizeof(ushort_pair_t));
     double l1normX = 0.0;
     double trSX = 0.0;
-    double logdetX = 0.0;
     for (unsigned long i = 0, k = 0; i < p ; i++, k += p) {
 	for (unsigned long j = 0; j < i; j++) {
 	    l1normX += Lambda[k+j]*fabs(X[k+j]);
@@ -300,8 +267,8 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 		for (unsigned long j = 0; j <= i; j++) {
 		    double g = S[k+j] - W[k+j];
 		    if (X[k+j] != 0.0 || (fabs(g) > Lambda[k+j])) {
-			activeSet[numActive].i = (unsigned short) i;
-			activeSet[numActive].j = (unsigned short) j;
+			activeSet[numActive].i = i;
+			activeSet[numActive].j = j;
 			numActive++;
 			if (X[k+j] > 0) 
 			    g += Lambda[k+j];
@@ -316,34 +283,32 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	    if (msg >= QUIC_MSG_NEWTON) {
 		MSG("Newton iteration %ld.\n", NewtonIter);
 		MSG("  Active set size = %ld.\n", numActive);
-		MSG("  sub-gradient = %e, l1-norm of X = %e.\n",
+		MSG("  sub-gradient = %le, l1-norm of X = %le.\n",
 		       subgrad, l1normX);
 	    }
 	    for (unsigned long cdSweep = 1; cdSweep <= 1 + NewtonIter/3;
 		 cdSweep++) {
 		diffD = 0.0;
-
 		for (unsigned long i = 0; i < numActive; i++ ) {
 		    unsigned long j = i + rand()%(numActive - i);
-		    unsigned short k1 = activeSet[i].i;
-		    unsigned short k2 = activeSet[i].j;
+		    unsigned long k1 = activeSet[i].i, k2 = activeSet[i].j;
 		    activeSet[i].i = activeSet[j].i;
 		    activeSet[i].j = activeSet[j].j;
 		    activeSet[j].i = k1;
 		    activeSet[j].j = k2;
 		}
 		for (unsigned long l = 0; l < numActive; l++) {
-		    unsigned long i = activeSet[l].i;
-		    unsigned long j = activeSet[l].j;
+		    int i = activeSet[l].i;
+		    int j = activeSet[l].j;
 		    CoordinateDescentUpdate(p, S, Lambda, X, W,
 					    U, D, i, j, normD,
 					    diffD);
 		}
 		if (msg >= QUIC_MSG_CD) {
-		    MSG("  Coordinate descent sweep %ld. norm of D = %e, "
-			   "change in D = %e.\n", cdSweep, normD, diffD);
+		    MSG("  Coordinate descent sweep %ld. norm of D = %le, "
+			   "change in D = %le.\n", cdSweep, normD, diffD);
 		}
-		if (diffD <= normD*cdSweepTol)
+		if (diffD < normD*cdSweepTol)
 		    break;
 	    }
 	}
@@ -352,22 +317,24 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	    // triangular part for the C arrays.
 	    ptrdiff_t info = 0;
 	    ptrdiff_t p0 = p;
-	    memcpy(U, X, sizeof(double)*p*p);
-	    dpotrf_((char*) "U", &p0, U, &p0, &info);
+	    memcpy(X1, X, sizeof(double)*p*p);
+	    dpotrf_((char*) "U", &p0, X1, &p0, &info);
 	    if (info != 0) {
 		MSG("Error! Lack of positive definiteness!");
 		iter[0] = -1;
 		free(activeSet);
 		free(U);
+		free(X1);
 		free(D);
 		if (pathLen > 1)
 		    free(Lambda);
 		return;
 	    }
+	    double detX = 0.0;
 	    for (unsigned long i = 0, k = 0; i < p; i++, k += (p+1))
-		logdetX += log(U[k]);
-	    logdetX *= 2.0;
-	    fX = (trSX + l1normX) - logdetX;
+		detX += log(X1[k]);
+	    detX *= 2.0;
+	    fX = -detX + trSX + l1normX;
 	}
 	double trgradgD = 0.0;
 	for (unsigned long i = 0, k = 0; i < p ; i++, k += p)
@@ -387,25 +354,23 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	    for (unsigned long i = 0, k = 0; i < p ; i++, k += p) {
 		for (unsigned long j = 0; j < i; j++) {
 		    unsigned long ij = k + j;
-		    W[ij] = X[ij] + D[ij]*alpha;
-		    l1normX1 += fabs(W[ij])*Lambda[ij];
-		    trSX1 += W[ij]*S[ij];
+		    X1[ij] = X[ij] + D[ij]*alpha;
+		    l1normX1 += fabs(X1[ij])*Lambda[ij];
+		    trSX1 += X1[ij]*S[ij];
 		}
 	    }
 	    l1normX1 *= 2.0;
 	    trSX1 *= 2.0;
 	    for (unsigned long i = 0, k = 0; i < p; i++, k += (p+1)) {
-		W[k] = D[k]*alpha + X[k];
-		l1normX1 += fabs(W[k])*Lambda[k];
-		trSX1 += W[k]*S[k];
+		X1[k] = D[k]*alpha + X[k];
+		l1normX1 += fabs(X1[k])*Lambda[k];
+		trSX1 += X1[k]*S[k];
 	    }
 	    // Note that the upper triangular part is the lower
 	    // triangular part for the C arrays.
 	    ptrdiff_t info = 0;
 	    ptrdiff_t p0 = p;
-	    // We are using W to hold the Cholesky decomposition; W
-	    // will hold the inverse later on.
-	    dpotrf_((char*) "U", &p0, W, &p0, &info);
+	    dpotrf_((char*) "U", &p0, X1, &p0, &info);
 	    if (info != 0) {
 		if (msg >= QUIC_MSG_LINE)
 		    MSG("    Line search step size %e.  Lack of positive "
@@ -413,51 +378,44 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 		alpha *= 0.5;
 		continue;
 	    }
-	    double logdetX1 = 0.0;
+	    double detX1 = 0.0;
 	    for (unsigned long i = 0, k = 0; i < p; i++, k += (p+1))
-		logdetX1 += log(W[k]);
-	    logdetX1 *= 2.0;
-	    fX1 = (trSX1 + l1normX1) - logdetX1;
+		detX1 += log(X1[k]);
+	    detX1 *= 2.0;
+	    fX1 = -detX1 + trSX1 + l1normX1;
 	    if (alpha == 1.0)
 		l1normXD = l1normX1;
-	    if (fX1 <= fX + alpha*sigma*(trgradgD + l1normXD - l1normX) ||
-		normD == 0) {
+	    if (fX1 < fX + alpha*sigma*(trgradgD + l1normXD - l1normX)) {
 		if (msg >= QUIC_MSG_LINE)
 		    MSG("    Line search step size chosen: %e.\n", alpha);
 		fXprev = fX;
 		fX = fX1;
 		l1normX = l1normX1;
-		logdetX = logdetX1;
-		trSX = trSX1;
 		break;
 	    }
-	    if (msg >= QUIC_MSG_LINE) {
-		MSG("    Line search step size %e.\n", alpha);
-		MSG("      Objective value would not decrease sufficiently: "
-		    "%e.\n", fX1 - fX);
-	    }
+	    if (msg >= QUIC_MSG_LINE)
+		MSG("    Line search step size %e.  Objective value "
+		       "would increase by %e.\n", alpha, fX1 - fX);
 	    if (fX1prev < fX1) {
 		fXprev = fX;
 		l1normX = l1normX1;
-		logdetX = logdetX1;
-		trSX = trSX1;
 		break;
 	    }
 	    fX1prev = fX1;
 	    alpha *= 0.5;
 	}
-	if (msg >= QUIC_MSG_NEWTON) {
+	if (msg >= QUIC_MSG_NEWTON)
 	    MSG("  Objective value decreased by %e.\n", fXprev - fX);
-	}
 	// compute W = inv(X):
 	ptrdiff_t info;
 	ptrdiff_t p0 = p;
-	dpotri_((char*) "U", &p0, W, &p0, &info);
-
+	dpotri_((char*) "U", &p0, X1, &p0, &info);
+	
 	for (unsigned long i = 0; i < p; i++) {
 	    for (unsigned long j = 0; j <= i; j++) {
-		double tmp = W[i*p+j];
+		double tmp = X1[i*p+j];
 		W[j*p+i] = tmp;
+		W[i*p+j] = tmp;
 	    }
 	}
 	for (unsigned long i = 0, k = 0; i < p; i++, k += p)
@@ -466,68 +424,46 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	if (mode == 'T') {
 	    if (opt != NULL)
 		opt[NewtonIter - 1] = fX;
-	    if (cputime != NULL) {
-		// Note that nn trace mode we do not want to count the
-		// time spent computing the duality gap.
-		cputime[NewtonIter - 1] = (clock()-timeBegin)/CLOCKS_PER_SEC;
-	    }
-	    if (dGap != NULL) {
-		double logdetW = projLogDet(p, S, W, U, Lambda);
-		double gap = -logdetW - p - logdetX + trSX + l1normX;
-		dGap[NewtonIter - 1] = gap;
-	    }
-	}
-	// Check for convergence.
-	if (subgrad*alpha >= l1normX*tol && (fabs((fX - fXprev)/fX) >= EPS))
-	    continue;
-	if (mode =='P') {
-	    if (opt != NULL)
-		opt[pathIdx] = fX;
-	    if (iter != NULL)
-		iter[pathIdx] = NewtonIter;
-	    if (dGap != NULL) {
-		double logdetW = projLogDet(p, S, W, U, Lambda);
-		double gap = -logdetW - p - logdetX + trSX + l1normX;
-		dGap[pathIdx] = gap;
-	    }
-	    for (unsigned long i = 0, k = 0; i < p; i++, k += p)
-		for (unsigned long j = i+1; j < p; j++)
-		    X[k+j] = X[j*p+i];
-	    double elapsedTime = (clock() - timeBegin)/CLOCKS_PER_SEC;
 	    if (cputime != NULL)
-		cputime[pathIdx] = elapsedTime;
-	    // Next lambda.
-	    pathIdx++;
-	    if (pathIdx == pathLen)
-		break;
-	    if (msg > QUIC_MSG_NO)
+		cputime[NewtonIter -1] = (clock()-timeBegin)/CLOCKS_PER_SEC;
+	}
+	if (subgrad*alpha < l1normX*tol
+	    || (fabs((fX - fXprev)/fX) < EPS)) {
+	    if (mode =='P') {
+		if (opt != NULL)
+		    opt[pathIdx] = fX;
+		iter[pathIdx] = NewtonIter;
+		for (unsigned long i = 0, k = 0; i < p; i++, k += p)
+		    for (unsigned long j = i+1; j < p; j++)
+			X[k+j] = X[j*p+i];
+		double elapsedTime = (clock() - timeBegin)/CLOCKS_PER_SEC;
+		if (cputime != NULL)
+		    cputime[pathIdx] = elapsedTime;
+		// Next lambda.
+		pathIdx++;
+		if (pathIdx == pathLen)
+		    break;
 		MSG("  New scaling value: %e\n", path[pathIdx]);
-	    unsigned long p2 = p*p;
-	    memcpy(X + p2, X, p2*sizeof(double)); 
-	    memcpy(W + p2, W, p2*sizeof(double)); 
-	    X += p2;
-	    W += p2;
-	    for (unsigned long i = 0; i < p*p; i++)
-		Lambda[i] = Lambda0[i]*path[pathIdx];
-	    continue;
+		unsigned long p2 = p*p;
+		memcpy(X + p2, X, p2*sizeof(double)); 
+		memcpy(W + p2, W, p2*sizeof(double)); 
+		X += p2;
+		W += p2;
+		for (unsigned long i = 0; i < p*p; i++)
+		    Lambda[i] = Lambda0[i]*path[pathIdx];
+		continue;
+	    }
+	    break;
 	}
-	break;
     }
-    if (mode == 'D') {
+    if (mode == 'D' || mode == 'T') {
+	iter[0] = NewtonIter;
 	if (opt)
 	    opt[0] = fX;
-	if (dGap != NULL) {
-	    double logdetW = projLogDet(p, S, W, U, Lambda);
-	    double gap = -logdetW - p - logdetX + trSX + l1normX;
-	    dGap[0] = gap;
-	}
-	if (iter != NULL)
-	    iter[0] = NewtonIter;
     }
-    if (mode == 'T' && iter != NULL)
-	iter[0] = NewtonIter - 1;
     free(activeSet);
     free(U);
+    free(X1);
     free(D);
     if (pathLen > 1)
 	free(Lambda);
@@ -535,20 +471,19 @@ void QUIC(char mode, uint32_t& p, const double* S, double* Lambda0,
 	for (unsigned long j = i+1; j < p; j++)
 	    X[k+j] = X[j*p+i];
     double elapsedTime = (clock() - timeBegin)/CLOCKS_PER_SEC;
-    if (mode == 'D' && cputime != NULL)
+    if (mode == 'D')
 	cputime[0] = elapsedTime;
-    if (msg >= QUIC_MSG_MIN)
+    if (msg >= QUIC_MSG_TIME)
 	MSG("QUIC CPU time: %.3f seconds\n", elapsedTime);
 }
 
 extern "C"
-void QUICR(char** modePtr, uint32_t& p, const double* S, double* Lambda0,
+void QUICR(char** modeptr, uint32_t& p, const double* S, double* Lambda0,
 	   uint32_t& pathLen, const double* path, double& tol,
-	   int32_t& msg, uint32_t& maxIter,
-	   double* X, double* W, double* opt, double* cputime,
-	   uint32_t* iter, double* dGap)
+	   int32_t& msg, uint32_t* iter, double* X,
+	   double* W, double* opt, double* cputime)
 {
-    char mode = **modePtr;
-    QUIC(mode, p, S, Lambda0, pathLen, path, tol, msg, maxIter, X, W,
-	 opt, cputime, iter, dGap);
+    char mode = **modeptr;
+    QUIC(mode, p, S, Lambda0, pathLen, path, tol, msg, iter, X, W,
+	 opt, cputime);
 }
